# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import calendar
import gettext
import os.path
import traceback
from itertools import chain
from datetime import datetime

import unohelper
from com.sun.star.lang import Locale
from com.sun.star.task import XJob
from com.sun.star.beans import PropertyValue
from com.sun.star.awt.WindowClass import TOP
from com.sun.star.awt.PosSize import POS, POSSIZE
from com.sun.star.awt.WindowAttribute import BORDER, CLOSEABLE
from com.sun.star.awt import (Rectangle, WindowDescriptor, XKeyListener, XMouseListener,
                                XTopWindowListener, XActionListener, XEnhancedMouseClickHandler)


IMPLEMENTATION_NAME = "jmz.calc.calendarpopup"

#-------------------
# UI locale strings
#-------------------
UI_STRINGS = {}
def loadstrings(ctx):
    def pgettext(domain, context, message):
        try:
            T = gettext.translation(domain, resource_path, [language])
            txt = T.gettext( '{}\x04{}'.format(context, message))
            if txt.startswith(context):   # domain exists but the string does not
                return txt[len(context)+1:]
            return txt
        except FileNotFoundError:
            return message
    try:
        smgr = ctx.ServiceManager
        ps = smgr.createInstance("com.sun.star.util.PathSubstitution")
        language = ps.getSubstituteVariableValue("vlang").replace("-", "_")
        program_path = unohelper.fileUrlToSystemPath(ps.getSubstituteVariableValue("prog"))
        resource_path = os.path.join(program_path, "resource")
        # 'today' translation
        UI_STRINGS['today'] = pgettext('sc', 'STR_COND_TODAY', 'today')
        ## 'calendar' translation
        # UI_STRINGS['calendar'] = pgettext('svx', 'RID_GALLERYSTR_THEME_CALENDAR', 'Calendar')
        # days and months translation
        alang = language.split("_") + 2*[""]
        locale = Locale(*alang[:3])
        localecalendar = smgr.createInstance("com.sun.star.i18n.LocaleCalendar")
        localecalendar.loadDefaultCalendar(locale)    # is "gregorian" always the default?
        UI_STRINGS['months'] = [""]+[m.AbbrevName for m in localecalendar.Months]
        UI_STRINGS['firstday'] = f = localecalendar.FirstDayOfWeek
        days = localecalendar.Days2[f:]+localecalendar.Days2[:f]
        UI_STRINGS['days'] = [d.NarrowName for d in days]
        print("calCalc UI strings:\n{}".format(UI_STRINGS))
    except:
        traceback.print_exc()

#---------------------
# listener base class
#---------------------
class Listener(unohelper.Base):
    def __init__(self):
        pass
    def disposing(self, source):
        pass

#----------
# Calendar
#----------
class TopWindowListener(Listener, XTopWindowListener):
    def windowClosing(self, event):
        # print ("windowClosing")
        dialog = event.Source
        dialog.setVisible(False)
        # dialog.dispose()
    def windowDeactivated(self, event):
        # print("windowDeactivated")
        dialog = event.Source
        if dialog.isVisible():
            dialog.setVisible(False)
            # dialog.dispose()
        pass
    def windowActivated(self, event):
        pass
    def windowOpened(self, event):
        pass
    def windowClosed(self, event):
        pass
    def windowMinimized(self, event):
        pass
    def windowNormalized(self, event):
        pass


class ActionListener(Listener, XActionListener):
    def __init__(self, dlg):
        self.dlg = dlg
    def actionPerformed(self, event):
        cmd = event.ActionCommand
        # print(cmd)
        try:
            offset = {'previous': -1, "next": 1}
            if cmd in offset and self.dlg.offsetcolors[0] == -1:    # default color -> monthly
                m = self.dlg.month + offset[cmd]
                self.dlg.month = (m-1) % 12 +1
                if m in (0, 13):
                    self.dlg.year += offset[cmd]
                self.dlg.loadcalendarcontent()
            elif cmd in offset:    # other color -> yearly
                self.dlg.year += offset[cmd]
                self.dlg.loadcalendarcontent()
            elif cmd == 'today':
                if self.dlg.month == self.dlg.today.month:
                    self.dlg.calendar.setVisible(False)
                    date = self.dlg.today.isoformat()[:10]
                    self.dlg.setdate(date)
                else:
                    self.dlg.month = self.dlg.today.month
                    self.dlg.year = self.dlg.today.year
                    self.dlg.loadcalendarcontent()
        except:
            traceback.print_exc()


class KeyListener(Listener, XKeyListener):
    def keyPressed(self, event):
        pass
    def keyReleased(self, event):
        # print("keyReleased")
        if event.KeyCode == 1281:
            event.Source.setVisible(False)


class MouseListener(Listener, XMouseListener):
    def __init__(self, dlg):
        self.dlg = dlg
    def mousePressed(self, event):
        pass
    def mouseReleased(self, event):
        model = event.Source.Model
        if model.FontWeight == 151:
            model.TextColor = self.dlg.offsetcolors[1]
            self.dlg.offsetcolors.reverse()
        else:
            self.dlg.calendar.setVisible(False)
            date = event.Source.Model.HelpURL
            self.dlg.setdate(date)


class Calendar():
    BACKGROUND = 0xffffe6
    DAY_WIDTH = 18

    def __init__(self, ctx, doc):
        self.ctx = ctx
        self.doc = doc
        self.parentwin = doc.CurrentController.Frame.ContainerWindow
        self.smgr = ctx.ServiceManager
        self.toolkit = self.create("com.sun.star.awt.Toolkit")
        self.actionlistener = ActionListener(self)
        self.mouselistener = MouseListener(self)
        self.controls = {}
        self.calendar = None
        self.pycal = calendar.Calendar((UI_STRINGS['firstday']+6) % 7)  # python 0 day = Monday while Libo 0 day = Sunday :(
        self.dates = []
        self.today = datetime.today()
        self.month = self.today.month
        self.year = self.today.year
        self.offsetcolors = [-1, 0xff0033]   # default color = monthly, red = yearly 
        self.createwindow()

    def create(self, service):
        return self.smgr.createInstance(service)

    def show(self, X=10, Y=10):
        try:
            self.loadcalendarcontent()
            self.calendar.setPosSize(X, Y, 0, 0, POS)
            self.calendar.setVisible(True)
            # self.calendar.dispose()
        except:
            traceback.print_exc()

    def createwindow(self):
        try:
            w = self.DAY_WIDTH
            descriptor = WindowDescriptor()
            descriptor.Type = TOP
            descriptor.WindowServiceName = ""
            descriptor.ParentIndex = 0
            descriptor.Parent = self.parentwin
            descriptor.Bounds = Rectangle(0, 0, w*7+2*2, w*7+20+25)
            descriptor.WindowAttributes = BORDER | CLOSEABLE
            self.calendar = self.toolkit.createWindow(descriptor) #2 xPeer|xWindow
            self.calendar.addTopWindowListener(TopWindowListener())
            self.calendar.addKeyListener(KeyListener())
            self.calendar.setBackground(self.BACKGROUND)
            # self.calendar.setControlFont(self.calendar.FontDescriptors[300])
            # self.calendar.setTitle("")

            self.create_control('btnleft', 'Button', (0, 0, 20, 20),
                                ('FocusOnClick', 'Label', 'Repeat', 'Tabstop'),
                                (False, '◂', True, False), action="previous")
            self.create_control('btnright', 'Button', (w*7+2-20, 0, 20, 20),
                                ('FocusOnClick', 'Label','Repeat', 'Tabstop'),
                                (False, '▸', True, False), action="next")
            self.create_control('hui', 'Button', (0, w*7+20, w*7+2, 22),
                                ('FocusOnClick', 'FontHeight', 'Label', 'Tabstop'),
                                (False, 8, UI_STRINGS['today'], False), action="today")
            self.create_control('month', 'FixedText', (22, 0, w*7+2*2-2*22, 20),
                                    ('Align', 'Border', 'FocusOnClick', 'FontHeight', 'FontName', 'FontWeight', 'VerticalAlign'),
                                    (1, 0, False, 8, "DejaVu Sans Condensed", 151, 1), mouse=True)
            for n, j in enumerate(UI_STRINGS['days']):
                self.create_control(j, 'FixedText', (n*w+1, 20, w, w),
                                    ('Align', 'Border', 'FocusOnClick', 'FontName', 'FontWeight', 'Label', 'TextColor', 'VerticalAlign'),
                                    (1, 0, False, "DejaVu Sans Condensed", 150, j, 0x004586, 1))
            for d in range(42):
                q, r = divmod(d, 7)
                name = '{}'.format(d)
                self.create_control(name, 'FixedText', (r*w+1, q*w+20+w, w, w),
                                    ('Align', 'Border', 'FocusOnClick', 'FontHeight', 'FontName', 'VerticalAlign'),
                                    (1, 0, False, 8, "DejaVu Sans Condensed", 1), mouse=True)
        except:
            traceback.print_exc()

    def create_control(self, name, type_, possize=None, prop_names=None, prop_values=None, action='', mouse=False):
        try:
            type_ = "com.sun.star.awt.UnoControl{}Model".format(type_)
            model = self.create(type_)
            if prop_names and prop_values:
                model.setPropertyValues(prop_names, prop_values)
            ctrl = self.create(model.DefaultControl)
            ctrl.setModel(model)
            ctrl.setContext(self.calendar)
            ctrl.createPeer(self.toolkit, self.calendar)
            if possize:
                ctrl.setPosSize(possize[0], possize[1], possize[2], possize[3], POSSIZE)
            if action:
                ctrl.setActionCommand(action)
                ctrl.addActionListener(self.actionlistener)
            if mouse:
                ctrl.addMouseListener(self.mouselistener)
            self.controls[name] = ctrl
        except:
            traceback.print_exc()

    def loadcalendarcontent(self):
        # print("loadcalendarcontent")
        months = UI_STRINGS['months']
        self.controls['month'].Model.Label = '{} {}'.format(months[self.month], self.year)
        dates = self.pycal.monthdatescalendar(self.year, self.month)
        if len(dates) == 5:
            week = 1
            if self.month == dates[-1][-1].month:
                week = 0
            try:
                dates.append(self.pycal.monthdatescalendar(self.year, self.month+1)[week])
            except calendar.IllegalMonthError:  #we are in december, let's go to next year
                dates.append(self.pycal.monthdatescalendar(self.year+1, 1)[week])
        self.dates = tuple(chain.from_iterable(dates))
        for n, date in enumerate(self.dates):
            model = self.controls['{}'.format(n)].Model
            color = 0x0
            bold = 100
            if date.month != self.month:
                color = 0x888888
            else:
                if date.isoformat() == self.today.isoformat()[:10]:
                    color = 0x800000
                    bold = 150
            model.setPropertyValues(("FontWeight", "HelpURL", "Label", "TextColor"), (bold, date.isoformat(),'{}'.format(date.day), color))

    def setdate(self, date):
        cursel = self.doc.CurrentSelection
        if cursel.supportsService("com.sun.star.sheet.SheetCell"):
            cursel.setFormula(date)
            if cursel.NumberFormat == 0:
                cursel.NumberFormat = self.doc.NumberFormats.getStandardFormat(2, cursel.CharLocale)
        elif cursel.supportsService("com.sun.star.sheet.SheetCellRange"):
            self.setdate_multi(cursel, date)
        elif cursel.supportsService("com.sun.star.sheet.SheetCellRanges"):
            for crange in cursel:
                self.setdate_multi(crange, date)

    def setdate_multi(self, frange, date):
            dates = ((date,)*frange.Columns.Count,)*frange.Rows.Count
            frange.setFormulaArray(dates)
            for r in frange.UniqueCellFormatRanges:
                if r.NumberFormat == 0:
                    r.NumberFormat = self.doc.NumberFormats.getStandardFormat(2, r.CharLocale)

#------
# XJob
#------
class MouseClickHandler(Listener, XEnhancedMouseClickHandler):
    def __init__(self, ctx, doc):
        # print("MouseClickHandler.__init__()")
        self.ctx = ctx
        self.doc = doc
    def mousePressed(self, e):
        return True
    def mouseReleased(self, e):
        # print("MouseClickHandler.mouseReleased()")
        if not e.Target.supportsService("com.sun.star.sheet.SheetCell"):
            return True
        if (e.Buttons, e.ClickCount, e.Modifiers) == (2, 1, 2):
            try:
                frame = self.doc.CurrentController.Frame
                ismultiselection = self.doc.CurrentSelection.queryIntersection(e.Target.RangeAddress).Count
                if not ismultiselection:
                    dispatcher = self.ctx.ServiceManager.createInstance("com.sun.star.frame.DispatchHelper")
                    dispatcher.executeDispatch(frame, ".uno:GoToCell", "", 0, (PropertyValue("ToPoint", 0, e.Target.AbsoluteName, 0),))
                calendar = Calendar(self.ctx, self.doc)
                Y = e.Y + frame.LayoutManager.CurrentDockingArea.Height + frame.LayoutManager.CurrentDockingArea.Y + 16
                X = e.X + 31
                calendar.show(X, Y)
                return True
            except:
                traceback.print_exc()
        return True


class TheCalendar(unohelper.Base, XJob):
    def __init__(self, ctx):
        print("TheCalendar.__init__()")
        self.ctx = ctx
        desktop = self.createUnoService("com.sun.star.frame.Desktop")
        self.doc = desktop.CurrentComponent

    def createUnoService(self, service):
        return self.ctx.ServiceManager.createInstanceWithContext(service, self.ctx)

    def execute(self, args):
        # print(str(args))
        try:
            if self.doc:
                if not UI_STRINGS:
                    loadstrings(self.ctx)
                self.doc.CurrentController.addEnhancedMouseClickHandler(MouseClickHandler(self.ctx, self.doc))
        except:
            traceback.print_exc()

#--------------
# registration
#--------------
g_ImplementationHelper = unohelper.ImplementationHelper()
g_ImplementationHelper.addImplementation (TheCalendar, IMPLEMENTATION_NAME, (IMPLEMENTATION_NAME,))
